#!/usr/bin/env python3
import os
import sys
from config import Config
from plugins import SleepPlugin, SyncPlugin, WaitPlugin, URRTDEPlugin
from plugin_repository import PluginRepository
from program_executor import ProgramExecutor
from http_server import ControllerHTTPServer
import logging
import signal


class HTTPControl:

    def GET_status(self, path, data):
        return 200, "Very good!"

    def POST_terminate(self, path, data):
        return 200, "Will do sir!"

    def POST_continue(self, path, data):
        return 201, "Will do sir!"


def main():
    # init
    logging.basicConfig(
        stream=sys.stdout,
        format="%(asctime)s [%(levelname)s]: %(name)s: %(message)s",
        level=logging.DEBUG
    )
    http_server = ControllerHTTPServer(HTTPControl())
    http_server.start()

    compiler_repo = PluginRepository()
    compiler_repo.register_plugin(SleepPlugin)
    compiler_repo.register_plugin(SyncPlugin)
    compiler_repo.register_plugin(WaitPlugin)
    compiler_repo.register_plugin(URRTDEPlugin)

    # Example code:
    compiler_repo.load_plugin("sleep")
    compiler_repo.load_plugin("sync")
    compiler_repo.load_plugin("wait")
    compiler_repo.load_plugin("ur_rtde")
    program = []
    program.append(compiler_repo.get_compiler("sleep").compile(secs=2))
    program.append(compiler_repo.get_compiler("moveJ").compile([5.7386425805573555, -0.536165146212658, 1.6278685933351111, -2.661452576366153, -1.5683528658421044, 1.0096729722787197], 1.0, 4.0))
    program.append(compiler_repo.get_compiler("moveL").compile([-0.4, 0.1, -0.31, 3.142, 0, 0], 0.05, 0.75))
    program.append(compiler_repo.get_compiler("moveL").compile([-0.4, 0.1, -0.24, 3.142, 0, 0], 0.05, 0.75))
    program.append(compiler_repo.get_compiler("moveJ").compile([5.923472948343555, 0.032637657012293965, 0.2590068609959585, -0.2935643801854462, -2.7157323161031766, 4.71238898038469], 1.0, 4.0))
    program.append(compiler_repo.get_compiler("moveJ").compile([4.982042349817814, -0.5256931707006921, 1.620887276327134, -1.0993828958312282, -3.660653573132907, 5.271592472723674], 1.0, 4.0))
    program.append(compiler_repo.get_compiler("wait").compile())
    program.append(compiler_repo.get_compiler("sleep").compile(secs=3))
    program.append(compiler_repo.get_compiler("sync").compile(nodes=2, name="test"))
    program.append(compiler_repo.get_compiler("sleep").compile(secs=10))
    program.append(compiler_repo.get_compiler("sleep").compile(secs=10))

    # prepare execution
    executor = ProgramExecutor(program, loop=True)

    def handle_stop_signal(signum, frame):
        logging.warning(f"Signal {signum} received. Aborting execution!")
        executor.abort()

        # Should be possible to call only once
        signal.signal(signal.SIGINT, signal.SIG_IGN)
        signal.signal(signal.SIGTERM, signal.SIG_IGN)

    signal.signal(signal.SIGINT, handle_stop_signal)
    signal.signal(signal.SIGTERM, handle_stop_signal)

    # start execution
    executor.start()

    # End of execution
    executor.join()
    compiler_repo.close()
    http_server.shutdown()


if __name__ == '__main__':
    main()
